@@ -281,12 +281,12 @@ Quadril::operator=(const Quadril &q)
 // VIEWPORT MEMBER DEFINITIONS
 
 Viewport::Viewport() : viewRotation(3,3) {  // allocate storage plots
+  surf.array().reserve(twoDMesh);
   plot=new flt_mtrx[numPlot];
   for(size_t j=0;j<numPlot;j++) {
     plot[j].reserve(oneDMesh,2);
     plot[j].resize(0,0);
   }
-  surf.array().reserve(twoDMesh);
 }
 
 Viewport::~Viewport() 
@@ -356,8 +356,8 @@ Viewport::cplot()
           rpos=r;
           cpos=c;
         }
-        p(1)=(cpos-1.0)/(ref.csize()-1.0);  // calculate position of the pixel
-        p(2)=1.0 - (rpos-1.0)/(ref.rsize()-1.0);
+        p(1)=(rpos-1.0)/(ref.rsize()-1.0);  // calculate position of the pixel
+        p(2)=(cpos-1.0)/(ref.csize()-1.0);
         z=interpolate(p);              // interpolate the value
         ref(r,c)=(z-f.zo)/f.zm;         // rescale zo to 0 and zm to 1
         if(ref(r,c)<0) ref(r,c)=0;             // clip to [0,1]
@@ -393,8 +393,7 @@ Viewport::cplot()
   
     if(shading==1) {                           // output the bitmap
       osp->form("/picstr %d string def\n",cpix);
-      osp->form("%d %d 8 \n[%d 0 0 -%d 0 %d] \n",cpix,rpix,cpix,\
-  rpix,rpix);
+      osp->form("%d %d 8 \n[%d 0 0 -%d 0 %d] \n",cpix,rpix,cpix,rpix,rpix);
       *osp << "{currentfile picstr readhexstring pop} image\n";
       for(size_t r=1; r<=rpix; r++) {  // step over the bitmap (output in hex)
         for(size_t c=1; c<=cpix; c++)
@@ -535,8 +534,12 @@ Viewport::setRotation()
     float x=v(1),y=v(2),z=v(3),t=(float)sqrt(accumulate(v,0.0,np));
     flt_mtrx &r=viewRotation;
     if(t==0) {
+#ifdef EX_HANDLE
+      throw gError("setRotation(): viewPoint has zero length.");
+#else
       cerr << "setRotation(): viewPoint has zero length.\n";
       exit(1);
+#endif
     }
     v/=t;                             // normalize v
     float dp=sqrt(x*x+y*y);           // length of projection onto x-y plane
@@ -617,11 +620,10 @@ Viewport::frameSet2D()
   
     if(yreset==1) {    // check for default frame
       f.yo=1;
-      f.ym=surf.rsize()-1;
+      f.ym=surf.csize()-1;
     }
     step=f.ym/(f.yn-1.0);
     for(size_t j=1;j<=f.yn;j++) f.ytics(j)=f.yo+step*(j-1);
-  
     if(zreset==1) {    // check for default frame
       float mx=max(surf);         // calculate the max and min
       float mn=min(surf);
@@ -1101,8 +1103,12 @@ initPlot(Viewport& vp,T tmin,T tmax,size_t length,F f,size_t dash)
 
   T step=(T)((tmax-tmin)/(length-1.0));
   if(length>vp.oneDMesh) {
+#ifdef EX_HANDLE
+    throw gError("initPlot(): too many steps.");
+#else
     cerr << "initPlot(): too many steps.\n";
     exit(1);
+#endif
   }
   vp.plot[dash].resize(length,2);        // reset plot size
   for(size_t j=1;j<=length;j++,tmin+=step) {  // store points
@@ -1120,8 +1126,12 @@ initPlot(Viewport& vp,T tmin,T tmax,size_t length,F f,Fl filter,size_t dash)
 
   T step=(T)((tmax-tmin)/(length-1.0));
   if(length>vp.oneDMesh) {
+#ifdef EX_HANDLE
+    throw gError("initPlot(): too many steps.");
+#else
     cerr << "initPlot(): too many steps.\n";
     exit(1);
+#endif
   }
   vp.plot[dash].resize(length,2);        // reset plot size
   for(size_t j=1;j<=length;j++,tmin+=step) {  // store points
@@ -1140,8 +1150,12 @@ initImplPlot(Viewport& vp,T tmin,T tmax,size_t length,F f,M m,size_t dash)
 
   T step=(T)((tmax-tmin)/(length-1.0));
   if(length>vp.oneDMesh) {
+#ifdef EX_HANDLE
+    throw gError("initImplPlot(): too many steps.");
+#else
     cerr << "initImplPlot(): too many steps.\n";
     exit(1);
+#endif
   }
   vp.plot[dash].resize(length,2);        // reset plot size
   for(size_t j=1;j<=length;j++,tmin+=step) {  // store points
@@ -1159,8 +1173,12 @@ initImplPlot(Viewport& vp,T tmin,T tmax,size_t length,F f,M m,Fl filter,size_t d
 
   T step=(T)((tmax-tmin)/(length-1.0));
   if(length>vp.oneDMesh) {
+#ifdef EX_HANDLE
+    throw gError("initImplPlot(): too many steps.");
+#else
     cerr << "initImplPlot(): too many steps.\n";
     exit(1);
+#endif
   }
   vp.plot[dash].resize(length,2);        // reset plot size
   for(size_t j=1;j<=length;j++,tmin+=step) {  // store points
@@ -1179,8 +1197,12 @@ initParamPlot(Viewport& vp,T tmin,T tmax,size_t length,Fx fx,Fy fy,size_t dash)
 
   T step=(T)((tmax-tmin)/(length-1.0));
   if(length>vp.oneDMesh) {
+#ifdef EX_HANDLE
+    throw gError("initParamPlot(): too many steps.");
+#else
     cerr << "initParamPlot(): too many steps.\n";
     exit(1);
+#endif
   }
   (vp.plot[dash]).resize(length,2);        // reset plot size
   for(size_t j=1;j<=length;j++,tmin+=step) {  // store points
@@ -1198,8 +1220,12 @@ initParamPlot(Viewport& vp,T tmin,T tmax,size_t length,Fx fx,Fy fy,Fl filter,siz
 
   T step=(T)((tmax-tmin)/(length-1.0));
   if(length>vp.oneDMesh) {
+#ifdef EX_HANDLE
+    throw gError("initParamPlot(): too many steps.");
+#else
     cerr << "initParamPlot(): too many steps.\n";
     exit(1);
+#endif
   }
   vp.plot[dash].resize(length,2);        // reset plot size
   for(size_t j=1;j<=length;j++,tmin+=step) {  // store points
@@ -1218,8 +1244,12 @@ initSurfPlot(Viewport& vp,T xmin,T xmax,size_t xlen,T ymin,T ymax,size_t ylen,F
   T xstep=(T)((xmax-xmin)/(xlen-1.0));
   T ystep=(T)((ymax-ymin)/(ylen-1.0));
   if(xlen*ylen>vp.twoDMesh) {
+#ifdef EX_HANDLE
+    throw gError("initSurf(): too many steps.");
+#else
     cerr << "initSurf(): too many steps.\n";
     exit(1);
+#endif
   }
   vp.xscale=(float)((xmax-xmin)/(xlen-1.0));  // set axis scalings
   vp.yscale=(float)((ymax-ymin)/(ylen-1.0));
@@ -1243,8 +1273,12 @@ initSurfPlot(Viewport& vp,T xmin,T xmax,size_t xlen,T ymin,T ymax,size_t ylen,F
   T xstep=(T)((xmax-xmin)/(xlen-1.0));
   T ystep=(T)((ymax-ymin)/(ylen-1.0));
   if(xlen*ylen>vp.twoDMesh) {
+#ifdef EX_HANDLE
+    throw gError("initSurf(): too many steps.");
+#else
     cerr << "initSurf(): too many steps.\n";
     exit(1);
+#endif
   }
   vp.xscale=(float)((xmax-xmin)/(xlen-1.0));  // set axis scalings
   vp.yscale=(float)((ymax-ymin)/(ylen-1.0));
