@@ -17,44 +17,46 @@ License along with this distribution; if not, write to the Free Software
 Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#include<stddef.h>
+
 struct c_double_array {
-  long unsigned s_;
+  size_t s_;
   double *data_;
 };
 
 struct c_double_vector {
-  long unsigned s_;
+  size_t s_;
   double *data_;
 };
 
 struct c_double_matrix {
-  long unsigned r_;
-  long unsigned c_;
+  size_t r_;
+  size_t c_;
   double *data_;
 };
 
 struct c_complex_array {
-  long unsigned s_;
+  size_t s_;
   struct c_complex *data_;
 };
 
 struct c_complex_vector {
-  long unsigned s_;
+  size_t s_;
   struct c_complex *data_;
 };
 
 struct c_complex_matrix {
-  long unsigned r_;
-  long unsigned c_;
+  size_t r_;
+  size_t c_;
   struct c_complex *data_;
 };
 
-long unsigned 
-a_elem(long unsigned s); 
+size_t 
+a_elem(size_t s); 
 
-long unsigned 
-v_elem(long unsigned s); 
+size_t 
+v_elem(size_t s); 
 
-long unsigned 
-m_elem(long unsigned rows,long unsigned cols,long unsigned r,long unsigned c); 
+size_t 
+m_elem(size_t rows,size_t cols,size_t r,size_t c); 
 
