@@ -42,11 +42,13 @@ gauss_jordan(mMatrix<T,S> &opm,mMatrix<T2,S2>& a,Real tol,bool full = 1,size_t r
 // (default) and a separate matrix of vector (column) inhomogeneities,
 // which is the argument a.
   if(tol<=(Real)0.0) tol=GJ_TINY;
+  mVector<Real,allocator<Real> > scale(opm.rsize());  // implicit pivot scaling
+  for(size_t j=1;j<=opm.rsize();j++) scale(j)=1.0/sup(opm.row(j));
   mVector<size_t,allocator<size_t> > swp(opm.rsize());
   for(size_t j=1;j<=opm.rsize();j++) swp(j)=j;  // init row swap label array
   size_t lim=min(opm.rsize()-r+1,opm.csize()-c+1);
   for(size_t j=1;j<=lim;j++,r++,c++) {          // loop over necessary rows
-    part_piv(opm,r,c,swp,tol,1);             // bring the pivot to row r
+    part_piv(opm,scale,r,c,swp,tol);             // bring the pivot to row r
     if(a.rsize()>0) a.row(swp(r))/=opm(swp(r),c);      // normalize pivot and
     opm.row(swp(r))/=opm(swp(r),c);                    // the row containing it
     if(full==1) {         // zero the elements above the diagonal if required
@@ -66,37 +68,37 @@ gauss_jordan(mMatrix<T,S> &opm,mMatrix<T2,S2>& a,Real tol,bool full = 1,size_t r
   if(a.rsize()>0) permute(a,swp);
 }
 
-template<class T,class S,class S2,class Real> 
+
+template<class T,class S,class S2,class S3,class Real> 
 int 
-part_piv(mMatrix<T,S> &opm,size_t r,size_t c,mVector<size_t,S2>& swp,Real tol,bool impl=1)
+part_piv(mMatrix<T,S> &opm,mVector<Real,S2> &scale,size_t r,size_t c,mVector<size_t,S3>& swp,Real tol)
 {
-//   Implements partial pivoting (i.e., looks for pivot element only in rows
-// below the current element, (swp(r),c), then swaps that row with the 
+//  Implements implicit partial pivoting (i.e., looks for pivot element only in 
+// rows below the current element, (swp(r),c), then swaps that row with the 
 // current one in the index map).
-// The default is for implicit pivoting (i.e., the pivot is chosen as if the
-// max coefficient in each row is set to 1). The map of swapped indeces is
+// The algorithm is for implicit pivoting (i.e., the pivot is chosen as if the
+// max coefficient in each row is set to 1) based on the scaling information
+// in the vector scale. The map of swapped indeces is
 // recorded in swp. The return value is +1 or -1 depending on whether the
 // number of row swaps was even or odd respectively. 
   if(tol<=(Real)0.0) tol=GJ_TINY;
   int swapNum=1;
-      // find the max element in each row
-  mVector<Real,allocator<Real> > max_elem(opm.rsize());
-  for(size_t j=r;j<=opm.rsize();j++) {        // calc. max_elem, current to bottom
-    if(!impl) max_elem(j)=(Real)1.0;      // if not implicit
-    else max_elem(j)=sup(opm.row(swp(j))); // else implicit pivoting
-  }
   size_t pivot=r;       // default pivot is the current position, [r,c]
-  Real piv_elem=abs(opm(swp(r),c))/max_elem(r);
+  Real piv_elem=abs(opm(swp(r),c))*scale(swp(r));
   for(size_t j=r+1;j<=opm.rsize();j++) {// loop over possible pivots below current
-    Real tmp=abs(opm(swp(j),c))/max_elem(j);  // if this elem is larger, 
+    Real tmp=abs(opm(swp(j),c))*scale(swp(j));  // if this elem is larger, 
     if(tmp>piv_elem) {                   // then it becomes the pivot
       pivot=j;
       piv_elem=tmp;
     }
   }
   if(piv_elem<tol) {
+#ifdef EX_HANDLE
+    throw gSingular(tol,"part_piv(): Zero pivot encountered.");
+#else
     cerr << "part_piv(): Zero pivot encountered.\n";
     exit(1);
+#endif
   }
   if(pivot>r) {         // bring the pivot to the diagonal
     size_t j=swp(r);                    // reorder swap array
@@ -119,13 +121,19 @@ lud(mMatrix<T,S> &opm,mVector<size_t,S2>& swp,Real tol)
 // number of row swaps was even or odd respectively.  swp must be 
 // preinitialized to a valid set of indices (e.g., {1,2, ... ,opm.rsize()}).
   if(tol<=(Real)0.0) tol=GJ_TINY;
+  mVector<Real,allocator<Real> > scale(opm.rsize());  // implicit pivot scaling
+  for(size_t j=1;j<=opm.rsize();j++) scale(j)=1.0/sup(opm.row(j));
   if(opm.rsize()!=opm.csize()) {
+#ifdef EX_HANDLE
+    throw gSize(opm.rsize(),opm.csize(),"lud(): Argument mMatrix is not square.");
+#else
     cerr << "lud(): Argument mMatrix is not square.\n";
     exit(1);
+#endif
   }
   int swapNum=1;
   for(size_t c=1;c<=opm.csize();c++) {             // loop over columns
-    swapNum*=part_piv(opm,c,c,swp,tol,1);       // bring pivot to diagonal
+    swapNum*=part_piv(opm,scale,c,c,swp,tol);       // bring pivot to diagonal
     for(size_t r=1;r<=opm.rsize();r++) {           //  loop over rows
       size_t lim=min(r,c);
       for(size_t j=1;j<lim;j++)
@@ -342,8 +350,12 @@ col_accum(const mMatrix<T,S> &opm,mMatrix<T2,S2> &q,size_t r,size_t c)
 // length opm.rsize(), and it is effectively padded with (r-1) leading zeroes.
 
   if(&opm==&q) {
+#ifdef EX_HANDLE
+    throw gError("col_accum(): Source and destination arrays the same.");
+#else
     cerr << "col_accum(): Source and destination arrays the same.";
     exit(1);
+#endif
   }
   for(c=c;c>=1;c--,r--) {     // loop over cols, deepest first
     mVector<T,allocator<T> > v(opm.rsize()-r+1);
@@ -367,8 +379,12 @@ row_accum(const mMatrix<T,S> &opm,mMatrix<T2,S2> &q,size_t r,size_t c)
 // length opm.csize(), and it is effectively padded with (c-1) leading zeroes.
 
   if(&opm==&q) {
+#ifdef EX_HANDLE
+    throw gError("row_accum(): Source and destination arrays the same.");
+#else
     cerr << "row_accum(): Source and destination arrays the same.\n";
     exit(1);
+#endif
   }
   for(r=r;r>=1;c--,r--) {     // loop over rows, deepest first
     mVector<T,allocator<T> > v(opm.csize()-c+1);
@@ -388,12 +404,20 @@ qrd(mMatrix<T,S> &opm,mMatrix<T,S2> &q)
 // transformation matrix.  The original matrix may be reconstructed as
 // dot(q,opm,opm).
   if(q.rsize()!=opm.rsize()) {
+#ifdef EX_HANDLE
+    throw gSize(q.rsize(),opm.rsize(),"qrd(): Mismatched matrices.");
+#else
     cerr << "qrd(): Mismatched matrices.\n";
     exit(1);
+#endif
   }
   if(opm.rsize()<opm.csize()) {
+#ifdef EX_HANDLE
+    throw gSize(opm.rsize(),opm.csize(),"qrd(): Underdetermined matrix.");
+#else
     cerr << "qrd(): Underdetermined matrix.";
     exit(1);
+#endif
   }
   for(size_t c=1;c<=opm.csize();c++) col_red(opm,c,c,1);  // find the H. vecs
   col_accum(opm,q,opm.csize(),opm.csize()); // accumulate orthog matrix
@@ -409,8 +433,12 @@ hessenberg(mMatrix<T,S> &opm,bool record = 1)
 // transformations. If record=0 then the lower triangle remains zero, otherwise
 // it is overwritten with the nontrivial parts of the Householder vectors.
   if(opm.rsize()!=opm.csize()) {
+#ifdef EX_HANDLE
+    throw gSize(opm.rsize(),opm.csize(),"hessenberg(): Non-square matrix.");
+#else
     cerr << "hessenberg(): Non-square matrix.\n";
     exit(1);
+#endif
   }
   for(size_t i=1;i<=opm.rsize()-2;i++) {
     col_red(opm,i+1,i,1);  // Reduce elements below subdiagonal and store H. vec
@@ -432,8 +460,12 @@ bidiag(mMatrix<T,S> &opm,bool record = 1)
 // Householder transformations.  Note that the bidiagonal matrix is NOT
 // similar to the original matrix.
   if(opm.csize()>opm.rsize()) {
+#ifdef EX_HANDLE
+    throw gSize(opm.rsize(),opm.csize(),"bidiag(): Underdetermined matrix.");
+#else
     cerr << "bidiag(): Underdetermined matrix.";
     exit(1);
+#endif
   }
   for(size_t i=1;i<=opm.csize()-2;i++) {
     col_red(opm,i,i,record);
@@ -517,8 +549,12 @@ row_givens(mMatrix<T,S> &opm,size_t r,size_t c,size_t r2)
 {
 // Zeroes opm(r,c) by rotating rows r and r2 (default c).
   if(r==r2) {
+#ifdef EX_HANDLE
+    throw gError("row_givens(): Bad row request.");
+#else
     cerr << "row_givens(): Bad row request.\n";
     exit(1);
+#endif
   }
   T si,co;
   if(r>r2) {
@@ -540,8 +576,12 @@ col_givens(mMatrix<T,S> &opm,size_t r,size_t c,size_t c2)
 {
 // Zeroes opm(r,c) by rotating columns c and c2 (default r).
   if(c==c2) {
+#ifdef EX_HANDLE
+    throw gError("col_givens(): Bad column request.");
+#else
     cerr << "col_givens(): Bad column request.\n";
     exit(1);
+#endif
   }
   T si,co;
   if(c>c2) {
@@ -557,11 +597,97 @@ col_givens(mMatrix<T,S> &opm,size_t r,size_t c,size_t c2)
   }
 }
 
+template<class Complex,class S1,class S2,class Real>
+Complex
+eigen_power(mMatrix<Complex,S1> &opm,Complex &cur,mVector<Complex,S2> &q,Real tol,unsigned itMax=SVD_IT_MAX)
+{
+// Computes largest eigenvalue via power iteration.  q must be
+// nonzero to start, but is otherwise unconstrained (more or less).
+  if(tol<=(Real)0.0) tol=SVD_TINY;
+  unsigned n=opm.rsize();
+  mVector<Complex,allocator<Complex> > z(n);
+  Complex prev,tmp1,tmp2;
+  q/=(Complex)sqrt(norm(q));                  // normalized eigenvector
+  Complex (*cp)(const Complex&)=conj;           // pointer to conjugate function
+  for(unsigned iter=1;iter<=itMax;iter++) {
+    dot(opm,q,z);                    // compute multiple z=A.q, new eigenvector
+    prev=cur;                      // store last guess
+    cur=sum(apply(q,cp)*z);        // new guess lambda=q^H.(A.q)
+    q=z;
+    q/=(Complex)sqrt(norm(q));             // new normalized eigenvector
+    if(abs(cur-prev)/abs(cur)<tol) // break if eigenvalue is not changing
+      return cur; 
+  }
+#ifdef EX_HANDLE
+  throw gIteration(itMax,"eigen_power(): No convergence.");
+#else
+  cerr << "eigen_power(): No convergence.\n";
+  exit(1);
+#endif
+}
+
+template<class Complex,class S1,class S2,class S3,class S4,class S5,class Real>
+Complex
+eigen_invit(mMatrix<Complex,S1> &opm,Complex &eval,mVector<Complex,S2> &evec,
+mMatrix<Complex,S3> &u,mVector<Real,S4> &sv,mMatrix<Complex,S5> &v,
+Real tol,unsigned itMax=SVD_IT_MAX)
+{
+// Computes closest eigenvalue to shift via inverse iteration. q must 
+// be nonzero to start, but is otherwise unconstrained (more or less).
+  Complex shift=eval;
+  opm.diagonal()-=shift;            // shift the diagonal
+  u=(Complex)0.0;
+  u.diagonal()=(Complex)1.0;
+  v=u;
+  pseudo_inverse(opm,u,sv,v,tol,itMax);  // compute the inverse shifted matrix
+  eigen_power(opm,eval,evec,tol,itMax);  // compute the eigenpair of the inverse
+  eval=shift + ((Complex)1.0)/eval; // invert and shift the the eigenvalue
+  return eval;
+}
+
+template<class Complex,class S1,class S2,class Real>
+Complex
+eigen_invit(mMatrix<Complex,S1> &opm,Complex &eval,mVector<Complex,S2> &evec,
+Real tol,unsigned itMax=SVD_IT_MAX)
+{
+// Same as eigen_invit() above but allocates its own temporaries
+  mMatrix<Complex,allocator<Complex> > u(opm.rsize(),opm.csize());
+  mMatrix<Complex,allocator<Complex> > v(opm.csize(),opm.csize());
+  mVector<Real,allocator<Real> > sv(opm.csize());
+  u=(Complex)0.0;
+  v=(Complex)0.0;
+  v.diagonal()=(Complex)1.0;
+  u.diagonal()=(Complex)1.0;
+  return eigen_invit(opm,eval,evec,u,sv,v,tol,itMax);
+}
+
 template<class Complex,class S,class Real> 
 void
-eigenvalues(mMatrix<Complex,S> &opm,Real tol)
+eigenvalues(mMatrix<Complex,S> &opm,Real tol,unsigned itMax=SVD_IT_MAX)
 {
 // Calculates the eigenvalues of a square (nonsymmetric) matrix via QR
+// Note that this algorithm will typically fail to converge for a real matrix.
+// opm is left in it's complex Schur form.  The eigenvalues are left on 
+// the diagonal of opm.
+  if(tol<=(Real)0.0) tol=SVD_TINY;
+  if(opm.rsize()!=opm.csize()) {
+#ifdef EX_HANDLE
+    throw gSize(opm.rsize(),opm.csize(),"eigenvalues(): Non-square matrix.");
+#else
+    cerr << "eigenvalues(): Non-square matrix.\n";
+    exit(1);
+#endif
+  }
+  hessenberg(opm,0); // reduce to hessenberg form, do not record House. vectors
+  eigenhess(opm,tol,itMax);   // compute the eigenvalues
+}
+
+
+template<class Complex,class S,class Real> 
+void
+eigenhess(mMatrix<Complex,S> &opm,Real tol,unsigned itMax=SVD_IT_MAX)
+{
+// Calculates the eigenvalues of a complex hessenberg matrix via QR
 // iteration with single shifts. Note that this algorithm will fail to
 // converge for a real matrix unless the matrix is Hermitian (real and
 // symmetric, since the eigenvalues are then real). opm is left in it's
@@ -569,14 +695,17 @@ eigenvalues(mMatrix<Complex,S> &opm,Real tol)
 // The eigenvalues are left on the diagonal of opm.
   if(tol<=(Real)0.0) tol=SVD_TINY;
   if(opm.rsize()!=opm.csize()) {
-    cerr << "eigenvalues(): Non-square matrix.\n";
+#ifdef EX_HANDLE
+    throw gSize(opm.rsize(),opm.csize(),"eigenhess(): Non-square matrix.");
+#else
+    cerr << "eigenhess(): Non-square matrix.\n";
     exit(1);
+#endif
   }
-  hessenberg(opm,0); // reduce to hessenberg form, do not record House. vectors
   size_t bottom=opm.rsize(),top;
   mVector<Complex,allocator<Complex> > v(2);        // the Householder vector
   v(1)=(Complex)1.0;
-  for(size_t k=0;k<=SVD_IT_MAX;k++) {  // QR iteration, first deflate the matrix
+  for(size_t k=0;k<=itMax;k++) {  // QR iteration, first deflate the matrix
     Real min=sup(opm);
     min=tol*tol*min*min;  // square of tol*largest element
     for(size_t i=bottom;i>=2;i--) {  // zero subdiagonals below tolerance
@@ -613,13 +742,17 @@ eigenvalues(mMatrix<Complex,S> &opm,Real tol)
     }    // j-loop
     for(size_t j=1;j<=opm.rsize();j++) opm(j,j)+=shift;
   }
+#ifdef EX_HANDLE
+  throw gIteration(itMax,"eigenvalues(): No convergence.");
+#else
   cerr << "eigenvalues(): No convergence.\n";
   exit(1);
+#endif
 }
 
 template<class T,class S,class S2,class Real,class S3,class S4> 
 mVector<Real,S3>& 
-svd(mMatrix<T,S>& opm,mMatrix<T,S2>& u,mVector<Real,S3> &d,mMatrix<T,S4>& v,Real tol)
+svd(mMatrix<T,S>& opm,mMatrix<T,S2>& u,mVector<Real,S3> &d,mMatrix<T,S4>& v,Real tol,unsigned itMax=SVD_IT_MAX)
 {
 // Returns the two orthogonal matrices and a vector of singular values. Thus
 // the COLUMNS of u and v are the left and right singular vectors of opm.
@@ -629,8 +762,12 @@ svd(mMatrix<T,S>& opm,mMatrix<T,S2>& u,mVector<Real,S3> &d,mMatrix<T,S4>& v,Real
 
   if(tol<=(Real)0.0) tol=(Real)SVD_TINY;
   if(opm.rsize()<opm.csize()) {
+#ifdef EX_HANDLE
+    throw gSize(opm.rsize(),opm.csize(),"svd(): Underdetermined matrix.");
+#else
     cerr << "svd(): Underdetermined matrix.";
     exit(1);
+#endif
   }
   size_t ortho=u.rsize();   // flag for accum. the orthog matrices
     // ortho=0 means u and v are null matrices and accum. is turned off
@@ -674,10 +811,14 @@ svd(mMatrix<T,S>& opm,mMatrix<T,S2>& u,mVector<Real,S3> &d,mMatrix<T,S4>& v,Real
   d(opm.csize())=real(opm(opm.csize(),opm.csize()));
   Real aa,bb,cc,c,s,mu;
   size_t bottom=opm.csize(), top=1;   // initially look at the bidiag matrix
-  for(size_t k=0;k<SVD_IT_MAX*opm.csize();k++) {
-    if(k==SVD_IT_MAX*opm.csize()) {
+  for(size_t k=0;k<itMax*opm.csize();k++) {
+    if(k==itMax*opm.csize()) {
+#ifdef EX_HANDLE
+      throw gIteration(itMax,"svd(): No convergence.");
+#else
       cerr << "svd(): No convergence.\n";
       exit(1);
+#endif
     }
     for(size_t i=1;i<=bottom-1;i++)  // search for zeroes on the super-diagonal
       if(abs(f(i))<=tol*(abs(d(i))+abs(d(i+1)))) f(i)-=f(i);
@@ -789,25 +930,25 @@ svd(mMatrix<T,S>& opm,mMatrix<T,S2>& u,mVector<Real,S3> &d,mMatrix<T,S4>& v,Real
 
 template<class T,class S,class Real,class S2> 
 mVector<Real,S2>& 
-singular_values(mMatrix<T,S> &opm,mVector<Real,S2> &sv,Real tol)
+singular_values(mMatrix<T,S> &opm,mVector<Real,S2> &sv,Real tol,unsigned itMax=SVD_IT_MAX)
 {
 // Same as svd(), but does not accumulate the left and right singular matrices.
   mMatrix<T,allocator<T> > u,v;
-  return svd(opm,u,sv,v,tol);
+  return svd(opm,u,sv,v,tol,itMax);
 }        // end singular_values()
 
 template<class T,class S,class S2,class Real,class S3,class S4>
 size_t
-pseudo_inverse(mMatrix<T,S> &opm,mMatrix<T,S2> &u,mVector<Real,S3> &sv,mMatrix<T,S4> &v,Real tol)
+pseudo_inverse(mMatrix<T,S> &opm,mMatrix<T,S2> &u,mVector<Real,S3> &sv,mMatrix<T,S4> &v,Real tol,unsigned itMax=SVD_IT_MAX)
 {
 // Finds the inverse restricted to the range in place.  u and v must be
 // identity matrices to start.  Left and right singular vectors are
 // the last columns of u and v respectively.
   if(tol<=(Real)0.0) tol=SVD_TINY;
-  svd(opm,u,sv,v,tol);     // take the svd of opm
-  if(sv(1)==(Real)0.0) {
-    cerr << "pseudo_inverse(): Zero matrix";
-    exit(1);
+  svd(opm,u,sv,v,tol,itMax);     // take the svd of opm
+  if(sv(1)==(Real)0.0) {   // opm is a zero matrix
+    opm=(T)0.0;
+    return 0;
   }
   u.transpose();           // the hermitian adjoint of u
   T (*cp)(const T&)=conj;
@@ -829,5 +970,20 @@ pseudo_inverse(mMatrix<T,S> &opm,mMatrix<T,S2> &u,mVector<Real,S3> &sv,mMatrix<T
   return urank;
 }
 
+template<class T,class S,class S2,class Real,class S3,class S4>
+size_t
+pseudo_inverse(mMatrix<T,S> &opm,Real tol,unsigned itMax=SVD_IT_MAX)
+{
+// Same as pseudo_inverse() above but allocates its own temporaries
+  mMatrix<Complex,allocator<Complex> > u(opm.rsize(),opm.csize());
+  mMatrix<Complex,allocator<Complex> > v(opm.csize(),opm.csize());
+  mVector<Real,allocator<Real> > sv(opm.csize());
+  u=(Complex)0.0;
+  v=(Complex)0.0;
+  v.diagonal()=(Complex)1.0;
+  u.diagonal()=(Complex)1.0;
+  return pseudo_inverse(opm,u,sv,v,tol,itMax);
+}
+
 #endif
 
